% ---------------------------------------------------------------------------
% The Subwoofer Room Impulse Response database (SUBRIR)
% analyseCalibration.m
% ---------------------------------------------------------------------------
% (c) 2016 KU LEUVEN - BANG & OLUFSEN A/S - AALBORG UNIVERSITY
%          FP7-PEOPLE Marie Curie Initial Training Network Dereverberation and Reverberation of Audio, Music, and Speech (DREAMS), funded by the European Commission under Grant Agreement no. 316969
% 		 http://www.dreams-itn.eu/index.php/dissemination/downloads/subrir
%          Giacomo Vairetti, giacomo.vairetti@esat.kuleuven.be
% ---------------------------------------------------------------------------
% 
% When using any portion of this data set, please cite:
% 
% G. Vairetti, N. Kaplanis, E. De Sena, S. H. Jensen, S. Bech, M. Moonen, and T. van Waterschoot, 
% The subwoofer room impulse response database (SUBRIR), KU Leuven, Tech. Rep., 2016.

%% Retrieves and analyze the calibration response measurements
% Retrieves the impulse response of the calibration measurements from the recorded signals, performs post-processing and shows a series of plots. 
% The user can specify which recorded signal to analyze (one of the 10
% recordings).
% 	The following plots are shown:
% 			- Magnitude spectrum of the retrieved RIR after post-processsing
% 			- Time response of the retrieved RIR after post-processsing
% 			- Spectrograms (recorded signal, retrieved NF impulse response before and after post-processing)
% 			- time and magnitude responses of the ESS and inverse signals

clear, clc, close all
head = '--------------------------------------------------------------------------- \n The Subwoofer Room Impulse Response database (SUBRIR) \n --------------------------------------------------------------------------- \n (c) 2016 KU LEUVEN - BANG & OLUFSEN A/S - AALBORG UNIVERSITY \n\n FP7-PEOPLE Marie Curie Initial Training Network Dereverberation \n and Reverberation of Audio, Music, and Speech (DREAMS), \n funded by the European Commission under Grant Agreement no. 316969 \n\n http://www.dreams-itn.eu/index.php/dissemination/downloads/subrir \n Giacomo Vairetti, giacomo.vairetti@esat.kuleuven.be \n --------------------------------------------------------------------------- \n When using any portion of this data set, please cite: \n G. Vairetti, N. Kaplanis, E. De Sena, S. H. Jensen, S. Bech, M. Moonen, and T. van Waterschoot, \n '' The subwoofer room impulse response database (SUBRIR),'' KU Leuven, Tech. Rep., 2016. \n \n Calibration measurements \n \n ';
fprintf(head)
load SUBRIR_Calibration
db =clb; clear clb
%% choose database
sub = questdlg('Choose Subwoofer', ...
	'SUBRIR database', ...
	'A - Genelec 1094','B - Genelec 7050B','B - Genelec 7050B');
switch sub
    case 'A - Genelec 1094'
        sb = 'A';
        raws = db.RAW_SpA;
        rir = db.RIR_SpA;
    case 'B - Genelec 7050B'
        sb = 'B';
        raws = db.RAW_SpB;
        rir = db.RIR_SpB;
end
%% choose recording number
prompt = {'recording number ([1,2]):'};
dlg_title = 'SUBRIR Calibration ';
num_lines = 1;
defaultans = {'1'};
answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
rec = str2double(answer{1}); %mic position

%%
name = strcat('SUBRIR Calibration - ',' Subwoofer  ',' ', sb);
sweep = db.Chirp;
invFilt = db.inverseFilter;
fs = db.samplingRate;
T = db.Duration;
silence = db.Silence;
latency = 880;
fn = fs/2;
%% retrieve RIR
%  Linear convolution
nDft = 2*(T+silence)*fs-1;
nDft2 = round(nDft/2);
recSignal = raws;
recSignal = permute(recSignal,[2,1]);
dftRecSignal = fft(recSignal,nDft);%/sqrt(nDft);
dftInvFilt = fft(invFilt,nDft);%*sqrt(nDft);
dftInvFiltMat = repmat(dftInvFilt,1,size(recSignal,2));
fullRIRs = (ifft(dftRecSignal.*dftInvFiltMat));
%% post-processing
% synchronous average
meanRIRfull = mean(fullRIRs,2);
% remove anti-causal part and latency
meanRIR = meanRIRfull(end-(T+silence)*fs+latency+1:end);
% set length to 5 s
meanRIR = meanRIR(1:5*fs);
% low-pass filtering
F=[1000 1100];  % band limits
A=[1 0];        % band type: 0='stop', 1='pass'
dev=[10^(0.1/20)-1 0.0001]; % ripple/attenuation spec
[M,Wn,beta,typ]= kaiserord(F,A,dev,fs); % window parameters
b=fir1(M,Wn,typ,kaiser(M+1,beta),'noscale'); % filter design
RIRlp = filtfilt(b,1,meanRIR); % filtfilt preserves delay
%%
h = RIRlp;
hfull = meanRIRfull;
N = length(h);

%% analysis RIR (time and frequency)
figure(1)
set(gcf,'Position',[100 100 1200 800])

% magnitude spectrum (log. frequency)
H =abs(fft(h,N));
subplot(2,2,1), semilogx(0:fs/N:fs/2-1/fs,20*log10(H(1:N/2)))
axis([0 12000 -60 Inf])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
zoom xon
title(strcat(name,'- Mag. Spectrum'))

% magnitude spectrum (zoom - lin. frequency)
subplot(2,2,3), plot(0:fs/N:fs/2-1/fs,20*log10(H(1:N/2))), 
axis([0 200 -60 Inf])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
zoom xon
title(strcat(name,' - Mag. Spectrum (zoom LFs)'))

% RIR
subplot(2,2,2),plot(1/fs:1/fs:N/fs,h)
zoom xon
axis([0 5 min(h)-0.05 max(h)+0.05])
xlabel('Time (s)')
ylabel('Amplitude')
title(strcat(name,' - averaged LP RIR'))
% RIR (zoom)
subplot(2,2,4),plot(1/fs:1/fs:N/fs,h)
zoom xon
axis([0 0.75 min(h)-0.01 max(h)+0.01])
xlabel('Time (s)')
ylabel('Amplitude')
title(strcat(name,' - averaged LP RIR (zoom)'))

%% Spectrograms 
spgr = exist('spgrambw','file');
if spgr
figure(2)
set(gcf,'Position',[100 100 1200 800])

% recorded signals
subplot(2,2,1)
spgrambw(recSignal(:,rec) ,fs ,'piclf',20,[log10(1) log10(fn)],[-75,-15]);
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (kHz)','Fontsize',14)
title(strcat(name,' - recorded signal ',num2str(rec)))

%
t_start = 1; t_stop = 4;
index1 = t_start*fs+1:t_stop*fs;
subplot(2,2,3), spgrambw(recSignal(index1,rec) ,fs ,'piclf',20,[log10(1) log10(fn)],[-75,-15]);
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca;
ax.XTick = [0 1 2 3];
XTick = {'1','2','3'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' -  recorded signal ',num2str(rec),' (zoom)'))


% retrieved RIR Full
index2 = 3*fs+1:9*fs;
subplot(2,2,2), spgrambw(hfull(index2,1) ,fs ,'pilcf',20,[log10(1) log10(fn)],[-95,-35]);
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca;
ax.XTick = [1 2 3 4 5 6 7 8 9];
XTick = {'-2','-1','0','1','2'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' -  retrieved RIR (only synch. avg)'))


% retrieved RIR post-processed
subplot(2,2,4), spgrambw(h ,fs ,'pilcf',20,[log10(1) log10(fn)],[-95,-35]);
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca;
ax.XTick = [ 0 1 2 3 4];
XTick = {'0','1','2','3','4'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' - retrieved RIR (post-processed)'))

else
            disp('function spgrambw.m not in the path')
        disp('Requires the voicebox toolbox for the spectrograms (spgrambw.m )')
        disp('http://www.ee.ic.ac.uk/hp/staff/dmb/voicebox/voicebox.html')
        disp('Using MATLAB built-in spectrogram function instead (wrong axis)')
%%
        figure(2)
set(gcf,'Position',[100 100 1200 800])

% recorded signals
subplot(2,2,1)
%spgrambw(recSignal(:,rec) ,fs ,'piclf',20,[log10(1) log10(fn)],[-65 5]);
spectrogram(recSignal(:,rec) ,1024,950,[],fs,'yaxis')
ax = gca; ax.YScale = 'log';
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (kHz)','Fontsize',14)
title(strcat(name,' - recorded signal ',num2str(rec)))

%
t_start = 1; t_stop = 4;
index1 = t_start*fs+1:t_stop*fs;
subplot(2,2,3),% spgrambw(recSignal(index1,rec) ,fs ,'piclf',20,[log10(1) log10(fn)],[-65 5]);
spectrogram(recSignal(index1,rec) ,1024,950,[],fs,'yaxis')
ax = gca; ax.YScale = 'log';
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca;
ax.XTick = [0 1 2 3];
XTick = {'1','2','3'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' -  recorded signal ',num2str(rec),' (zoom)'))


% retrieved RIR Full
index2 = 3*fs+1:9*fs;
subplot(2,2,2), %spgrambw(hfull(index2,1) ,fs ,'pilcf',20,[log10(1) log10(fn)],[-93 -23]);
spectrogram(hfull(index2,1),1024,950,[],fs,'yaxis')
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca; ax.YScale = 'log';
ax.XTick = [1 2 3 4 5 6 7 8 9];
XTick = {'-2','-1','0','1','2'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' -  retrieved RIR (only synch. avg)'))


% retrieved RIR post-processed
subplot(2,2,4), %spgrambw(h ,fs ,'pilcf',20,[log10(1) log10(fn)],[-93 -23]);
spectrogram(h,1024,950,[],fs,'yaxis')
xlabel('Time (s)','Fontsize',14); ylabel('Log-scaled Frequency (Hz)','Fontsize',14)
ax = gca; ax.YScale = 'log';
ax.XTick = [ 0 1 2 3 4];
XTick = {'0','1','2','3','4'};
set(gca,'XTickLabel',XTick)
title(strcat(name,' - retrieved RIR (post-processed)'))
end
%% Chirp
spgr = exist('spgrambw','file');
if spgr
    figure(3),
    set(gcf,'Position',[100 100 1200 800])
    subplot(2,2,1),spgrambw(sweep,fs ,'piclfw',5,[0 log10(fn)],[-80 -30]);
    title('ESS - log-scaled frequency')
    subplot(2,2,3),spgrambw(sweep,fs ,'picfw',15,fn,[-80 -30]);
    title('ESS - linear frequency')
    subplot(2,2,2),spgrambw(invFilt,fs ,'piclfw',5,[0 log10(fn)],[-80 -30]);
    title('Inverse Filter - log-scaled frequency')
    subplot(2,2,4),spgrambw(invFilt,fs ,'picfw',15,fn,[-80 -30]);
    title('Inverse Filter - linear frequency')
else
    disp('!!!WARNING!!!')
    disp('function spgrambw.m not in the path')
    disp('Requires the voicebox toolbox for the spectrograms (spgrambw.m )')
    disp('http://www.ee.ic.ac.uk/hp/staff/dmb/voicebox/voicebox.html')
    disp('Using MATLAB built-in spectrogram function instead (wrong axis)')

    figure(3),
    set(gcf,'Position',[100 100 1200 800])
    subplot(2,2,1),spectrogram(sweep,1024,950,[],fs,'yaxis')
    ax = gca; ax.YScale = 'log';
    title('ESS - log-scaled frequency')
    subplot(2,2,3),spectrogram(sweep,1024,950,[],fs,'yaxis')
    ax = gca; ax.YScale = 'log';
    title('ESS - linear frequency')
    subplot(2,2,2),spectrogram(invFilt,1024,950,[],fs,'yaxis')
    ax = gca; ax.YScale = 'log';
    title('Inverse Filter - log-scaled frequency')
    subplot(2,2,4),spectrogram(invFilt,1024,950,[],fs,'yaxis')
    ax = gca; ax.YScale = 'log';
    title('Inverse Filter - linear frequency')    
end
%%
freqVector = fs*(0:nDft-1)'/nDft;
dftSweep = fft(sweep,nDft);
figure(4),
semilogx(freqVector,20*log10([...
    abs(dftSweep),...
    abs(dftInvFilt),...
    abs(dftSweep.*dftInvFilt)]),'LineWidth',1.5,'Color','k')
xlabel('Log-scaled Frequency (Hz)','Fontsize',13)
ylabel('Magnitude (dB)','Fontsize',13)
axis([0.1,24000,-50,50])
XTick = {'0.1','1','10','100','1k','10k'};
set(gca,'XTickLabel',XTick)
hold off
title('sweep, inverse filter & convolution result (mag.resp)')